#include <windows.h>
#include <GL/gl.h>
#include "glext.h"
#include "shader.h"

extern "C"
{
	int _fltused = 0;
}

//-----------------------------------------------------------------------------
// fexp
//-----------------------------------------------------------------------------
__forceinline float fexp(float v)
{
	volatile float res;
	__asm
	{
		fld v;
		fldl2e;
		fmulp st(1), st;
		fld1;
		fld   st(1);
		fprem;
		f2xm1;
		faddp st(1), st;
		fscale;
		fstp  st(1);
		fstp res;
	}
	return res;
}

///////////////////////////////////////////////////////////////////
// WINDOWING
#define SCREENX 1680
#define SCREENY 1050

#pragma data_seg(".pfd")
PIXELFORMATDESCRIPTOR pfd={
sizeof(PIXELFORMATDESCRIPTOR),1,PFD_DRAW_TO_WINDOW |PFD_SUPPORT_OPENGL |PFD_DOUBLEBUFFER,PFD_TYPE_RGBA,
32,0, 0, 0, 0, 0, 0,0,0,0,0, 0, 0, 0,32,0,0,PFD_MAIN_PLANE,0,0, 0, 0};

#pragma data_seg(".dms")
DEVMODE dmScreenSettings={
L"",0,0,sizeof(dmScreenSettings),0,DM_PELSWIDTH|DM_PELSHEIGHT,
0,0,0,0,0,0,0,0,0,0,0,0,0,L"",0,0,SCREENX,SCREENY,0,0,0,0,0,0,0,0,0,0
};
 
#pragma code_seg(".compile")
__forceinline void compileShader()
{
	GLuint s,p;
	    
    p = ((PFNGLCREATEPROGRAMPROC)wglGetProcAddress("glCreateProgram"))();
		
	s = ((PFNGLCREATESHADERPROC)(wglGetProcAddress("glCreateShader")))(GL_VERTEX_SHADER);
		((PFNGLSHADERSOURCEPROC)wglGetProcAddress("glShaderSource")) (s, 1, (const GLchar**)(&vsh), NULL);
	    ((PFNGLCOMPILESHADERPROC)wglGetProcAddress("glCompileShader"))(s);
	    ((PFNGLATTACHSHADERPROC)wglGetProcAddress("glAttachShader")) (p,s);
		
    s = ((PFNGLCREATESHADERPROC)wglGetProcAddress("glCreateShader"))(GL_FRAGMENT_SHADER);	
		((PFNGLSHADERSOURCEPROC)wglGetProcAddress("glShaderSource")) (s, 1, (const GLchar**)(&fsh), NULL);
		((PFNGLCOMPILESHADERPROC)wglGetProcAddress("glCompileShader"))(s);
		((PFNGLATTACHSHADERPROC)wglGetProcAddress("glAttachShader")) (p,s);

		((PFNGLLINKPROGRAMPROC)wglGetProcAddress("glLinkProgram"))(p);
		((PFNGLUSEPROGRAMPROC)wglGetProcAddress("glUseProgram"))(p);
}


//////////////////////////////////////////////////////////////////
//  MAIN

#pragma code_seg(".main")
void WINAPI WinMainCRTStartup()
{
	ChangeDisplaySettings (&dmScreenSettings,CDS_FULLSCREEN); 
	HDC hDC = GetDC( CreateWindow(L"edit", 0, WS_POPUP|WS_VISIBLE|WS_MAXIMIZE, 0, 0, 0, 0, 0, 0, 0, 0) );
	SetPixelFormat ( hDC, ChoosePixelFormat ( hDC, &pfd) , &pfd );
	wglMakeCurrent ( hDC, wglCreateContext(hDC) );
	ShowCursor(FALSE); 
	compileShader();

	int start = GetTickCount();
	float t = 0.0f;
	float xc = -1.00945f;
	float yc = 0.37540f;

	while(t < 10.0f && !GetAsyncKeyState(VK_ESCAPE))
	{
		t=(float)(GetTickCount()-start) * 0.001f;
		float ax = 2.0f*fexp(-t);
		float ay = ax*SCREENY/float(SCREENX);

		glBegin(GL_QUADS);
			glTexCoord2f(xc+ax, yc+ay);  glVertex2f(1.0,1.0);
			glTexCoord2f(xc-ax, yc+ay);  glVertex2f(-1.0,1.0);
			glTexCoord2f(xc-ax, yc-ay);  glVertex2f(-1.0,-1.0);
			glTexCoord2f(xc+ax, yc-ay);  glVertex2f(1.0,-1.0);
		glEnd();
		SwapBuffers(hDC);
	}

   ExitProcess(0);
}
